/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18, 5737-M66
 * (C) Copyright IBM Corp. 2019,2021 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication, or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

Polymer({
	is: 'maximo-inventory-picklist-detail',
	behaviors: [BaseComponent],
	listeners: {
		'date-changed': 'editRequiredDate',
		'maximo-lookup-selected' : '_returnFromLookup',
		'collection-count-change': '_setTabCount',
		'selected-table-row':'_selectBatchAction',
        'tableselectall-clicked':'_selectBatchAction'
	},
	properties : {		
    	record: {
    		type: Object,
  			notify: true
		},
		collection : {
			type : Object,
			notify: true
		},
		schema : {
			type : Object
		},
    	plrecord: {
    		type: Object,
    		notify: true
    	},
		attributeList: {
			type: String,
			value: 'picklistnum,description,fromstoreloc,requireddate--reqdatepicklist,createddate,status'
		},
		dateformat: {
			type: String,
			value: function(){
				if($M.isMobile()){
					return 'YYYY-MM-DD';
				} else if($M.maxauth && $M.maxauth.whoami){
					return $M.maxauth.whoami.dateformat.toUpperCase();
				} else {
					return 'M/D/YY';
				}
			}
		},
		openqtyColumn: {
			type: Array,
			value: []
		},
		pickListTabName: {
    		type: String,
    		observer: '_loadSection'    		
    	},
		invuselineopencount: {
			type: Number,
			value: 0
		},
		invuselinepickedcount: {
			type: Number,
			value: 0
		},
		invuselineissuedcount: {
			type: Number,
			value: 0
		},
		/**
		 * Interval in seconds to undo delete operation
		 */
    	undoIntervalSecs: {
    		Type: Number,
    		value: 3,
    		readOnly: false
    	},
    	
    	_showMultiSelect: {
            type: Boolean,
            value:false
        }
	},
	
	handlePicklistInit: function(){
		this.async(function(){
			$j(this.$.plCreationDate).find('input').attr('readonly','readonly');
			if(this.plrecord){
				this.$.plCreationDate.value = moment(this.plrecord.createddate).format(this.dateformat);
				this.$.plRequiredDate.value = moment(this.plrecord.reqdatepicklist).format(this.dateformat);
			}
		},100);
		this._refreshPickedTabCount();
		this._refreshStagedTabCount();
		this._refreshIssuedTabCount();
		this._refreshReviewTabCount();
	},
	  
	/**
	 * Handles back button click
	 */
	_handlePLBack: function(e){
		this.pickListTabName = '';
		$M.setSubRoute('reservations/pickList');
		this.fire('display-itempage', {'pagename': 'reservation_list', 'tabName':'pickList'});
	},
	
	_applyEditDesc: function(e){
		if (this.$.textDescription.value !== undefined && this.$.textDescription.value.trim().length === 0) {
			this.$.textDescription.value = this.plrecord.description;
			return false;
		}
		var params = {
			'description' : this.$.textDescription.value
		};
		var pickListDesc = this.$.textDescription.value.trim();
		//Update the description if there is a change
		if(pickListDesc && this.plrecord && this.plrecord.description && pickListDesc.trim() !== this.plrecord.description.trim()){
			this._updatePLDetail(params);
		}
	},
	
	_updatePLDetail: function(params){
		var responseProperties = this.attributeList;		
		this.$.picklistresource.resourceUri = this.plrecord.href;

		$M.toggleWait(true);
		this.$.picklistresource.updateRecord(params, responseProperties, true);
	},
	
	_handleRecordUpdateError : function(e){
		$M.toggleWait(false);
		this._setPreEditValues();
		
		var error = e.detail;
		if (error && error.request && error.request.response && error.request.response.Error) {
			$M.notify(error.request.response.Error.message, $M.alerts.error);
		}
	},	

	_handleRecordUpdateSuccess : function(e){
		$M.toggleWait(false);
		var targetResource = e.target;
		if (targetResource && targetResource.__data__ && targetResource.__data__.resourceData) {
			this.plrecord = targetResource.__data__.resourceData;
		}
	},
	
	/**
	 * Reset previous values on error on tile save
	 */
	_setPreEditValues: function(){
		$M.toggleWait(false);
		
		var self = this;
		this.$.picklistresource.resourceUri = this.plrecord.href;
		this.$.picklistresource.loadRecord().then(function(result){
			if(result) {
				self.plrecord = result;
				self.$.textDescription.value = self.plrecord.description;
				self.$.plRequiredDate.value = moment(self.plrecord.reqdatepicklist).format(self.dateformat);
				self.$.labelPicklistStatus.label = self.plrecord.status_description;
			}				
		});
	},
	
	editRequiredDate: function(e){
		var reqdate = '';
		
		if(this.plrecord && this.plrecord.reqdatepicklist){
			reqdate = moment(this.plrecord.reqdatepicklist).format(this.dateformat);
		}		
		if(e.detail && e.detail.value && reqdate && (e.detail.value !== reqdate)){
			var params = {
				'requireddate' : moment(e.detail.value, this.dateformat).format('YYYY-MM-DDThh:mm:ss')
			};
			this._updatePLDetail(params);
		} else if(e.detail && e.detail.value === ''){
			this.$.plRequiredDate.value = moment(Date.parse(new Date())).format(this.dateformat);
		}
	},
	
	_showStatus : function() {
		$M.showLookup('pickliststatus', this,'pickliststatus', this.plrecord.href);
	},
	
	_returnFromLookup : function(e) {
		var self=this;
		if(e.detail && e.detail.records.length){
			if(e.detail.identifier==='reservationLookup'){
				var reservationlist = {};
				reservationlist.msr = {};
				reservationlist.msr.selection = [];
				var items = e.detail.records;
				items.forEach(function(record, index){
					reservationlist.msr.selection[index] = {};
					reservationlist.msr.selection[index].invreserveid = record.invreserveid;
		    	});
				
				reservationlist.pl_num= this.plrecord ? this.plrecord.picklistnum : undefined;
				reservationlist.siteid = ($M.getMaxauth().whoami.defaultSite) ? $M.getMaxauth().whoami.defaultSite : '';
		        this.$.picklistresource.updateAction('wsmethod:addReservation',reservationlist).then(
						function(result) {
							self.$.picklist_open.completeOverlayFunc(self.$.picklist_open);					
						}, function(error) {
							$M.showResponseError(error);										
						});
			}else{
				var statusdesc = e.detail.records[0].description;
				var params = {
					'status' : e.detail.records[0].value,
					'_action' : 'Change',
					'accessModifier': 1,
				};
				this.updatePLStatus(params, statusdesc);
			}
		}
	},
	
	updatePLStatus: function(params, statusdesc){
		var self = this;
		var updateStatusMsg = $M.localize('uitext', 'mxapiinvpicklist', 'updatestatusmsg', [statusdesc]);
    	var updateStatusAction = true;
    	var picklistresource = this.$.picklistresource;
    	var responseProperties = this.attributeList;
        picklistresource.resourceUri = this.plrecord.href + '?responseos=MXAPIINVPICKLIST';
        
        var undoCallback = function(){
    		$j(statusnotification).remove();
    		updateStatusAction = false;
    	};
    	
    	var statusnotification = $M.notify(updateStatusMsg, $M.alerts.info, null, undoCallback, false);
		
		this.async(function(){
			$j(statusnotification).remove();
	    	if(updateStatusAction){
	    		this.updateStatus(picklistresource, params, responseProperties, this);
	    	}
		}, (this.undoIntervalSecs*1000));
	},
	
	updateStatus: function(picklistresource, params, responseProperties, self){
		$M.toggleWait(true);
		picklistresource.updateAction('wsmethod:changeStatus', params, responseProperties).then(
	        	function(result){
	        		if(result.response) {
	        			$M.toggleWait(false);
	        			self.plrecord = result.response;
	        			if(self.plrecord.status_maxvalue === "CLOSED" || self.plrecord.status_maxvalue === "CANCELLED"){
	        				switch(self.pickListTabName){
	            			case 'open':{
	            				self.$.picklist_open._refreshCollection();
	            				self.$.picklist_open.$.takeActionLink.hidden=true;
	            			}
	            			    break;
	            			case 'picked':
	            				self.$$('#picklist_picked')._refreshCollection();
	            			    break;
	            			case 'issued':
	            				self.$$('#picklist_issued')._refreshCollection();
	            			    break;
	            			case 'staged':
	            				self.$$('#picklist_staged')._refreshCollection();
	            			    break;
	            	        default:
	            	        	self.$.picklist_open._refreshCollection();
	            	        	break;
	            			}
	        			}
	        			if(self.plrecord.status_maxvalue === "CANCELLED"){
	        				self._refreshReviewTabCount();
	        			}
	        		}
	        	});
	},
	
	_loadSection: function(plDtlTabName){
		var sectionid = plDtlTabName + '_section';
        $j(this.$[sectionid]).siblings().hide();
        $j(this.$[sectionid]).show();
		switch(plDtlTabName){
		case 'open':
			$M.setSubRoute('reservations/pickList/details/open');
			this.$.picklist_open._loadPickListOpenTable();
			this.$.picklist_content_selector.select(0);
		    break;
		case 'picked':
			this.addContent('picked_section');
			$M.setSubRoute('reservations/pickList/details/picked');
			this.$$('#picklist_picked')._loadPickListPickedTable();
			this.$.picklist_content_selector.select(1);
		    break;
		case 'issued':
			this.addContent('issued_section');
			$M.setSubRoute('reservations/pickList/details/issued');
			this.$$('#picklist_issued')._loadPickListIssuedTable();
			this.$.picklist_content_selector.select(3);
		    break;
		case 'staged':
			this.addContent('staged_section');
			$M.setSubRoute('reservations/pickList/details/staged');
			this.$$('#picklist_staged')._loadPickListStagedTable();
			this.$.picklist_content_selector.select(2);
		    break;
		case 'review':
			this.addContent('review_section');
			$M.setSubRoute('reservations/pickList/details/review');
			this.$$('#picklist_review')._loadPickListReviewTable();
			this.$.picklist_content_selector.select(4);
		    break;
        default:
        	$M.setSubRoute('reservations/pickList/details');
        	this.$.picklist_open._loadPickListOpenTable();
        	this.$.picklist_content_selector.select(0);
        	break;
		}
		if($j(window).scrollTop()){
            window.scrollTo(0,0);
        }
	},
	
	_setTabCount: function(e){
		if(e.detail.invuselineopencount !== undefined){
			this.invuselineopencount = e.detail.invuselineopencount;
		}
		if(e.detail.invuselinepickedcount !== undefined){
			this.invuselinepickedcount = e.detail.invuselinepickedcount;
		}
		if(e.detail.invuselinestagedcount !== undefined){
			this.invuselinestagedcount = e.detail.invuselinestagedcount;
		}
		if(e.detail.invuselineissuedcount !== undefined){
			this.invuselineissuedcount = e.detail.invuselineissuedcount;
		}
		switch(e.detail.action){
		case 'Pick':
			this._refreshPickedTabCount();
			break;
		case 'Stage':
			this._refreshStagedTabCount();
			break;
		case 'Issue':
			this._refreshIssuedTabCount();
			break;
		case 'Restock':
			this._refreshOpenTabCount();
			break;
		case 'Remove':
			this._refreshReviewTabCount();
			break;
		}
	},
	
	_handlePageNavigation: function(e){
    	var name = e.currentTarget.getAttribute('name');
    	this.selectTab(name);
	},
	/**
	 * Update selectedTabName property
	 */
	selectTab: function(tabName){
	    if (this.pickListTabName !== tabName) {
	        this.pickListTabName = tabName;
	    }
	},

	_refreshOpenTabCount: function(){
		var pickedResource = this.$.picklistresource;
		if(pickedResource) {
			var self = this;
			var additionalparams = [];
			additionalparams.push('oslc.where=openqty>0');
			pickedResource.additionalParams = additionalparams;
			pickedResource.resourceUri = this.plrecord.href+'/invuseline?count=1';
			pickedResource.loadRecord().then(function(result){
				self.invuselineopencount = result.totalCount;
			});
		}
	},

	_refreshPickedTabCount: function(){
		var pickedResource = this.$.picklistresource;
		if(pickedResource) {
			var self = this;
			var additionalparams = [];
			additionalparams.push('oslc.where=pickedqty>0');
			pickedResource.additionalParams = additionalparams;
			pickedResource.resourceUri = this.plrecord.href+'/invuseline?count=1';
			pickedResource.loadRecord().then(function(result){
				self.invuselinepickedcount = result.totalCount;
			});
		}
	},
	
	_refreshStagedTabCount: function(){
		var pickedResource = this.$.picklistresource;
		if(pickedResource) {
			var self = this;
			var additionalparams = [];
			additionalparams.push('oslc.where=stagedqty>0');
			pickedResource.additionalParams = additionalparams;
			pickedResource.resourceUri = this.plrecord.href+'/invuseline?count=1';
			pickedResource.loadRecord().then(function(result){
				self.invuselinestagedcount = result.totalCount;
			});
		}
	},
	
	_refreshIssuedTabCount: function(){
		var pickedResource = this.$.picklistresource;
		if(pickedResource) {
			var self = this;
			var additionalparams = [];
			additionalparams.push('oslc.where=issuedqty>0');
			pickedResource.additionalParams = additionalparams;
			pickedResource.resourceUri = this.plrecord.href+'/invuseline?count=1';
			pickedResource.loadRecord().then(function(result){
				self.invuselineissuedcount = result.totalCount;
			});
		}
	},
	
	_refreshReviewTabCount: function(){
		var pickedResource = this.$.picklistresource;
		if(pickedResource) {
			var self = this;
			pickedResource.additionalParams = [];
			pickedResource.resourceUri = this.plrecord.href+'/invuseline?count=1';
			pickedResource.loadRecord().then(function(result){
				self.invuselinereviewcount = result.totalCount;
			});
		}
	},
	
	_selectBatchAction: function(e){
		switch(this.pickListTabName){
		case 'open':
			this._multiSelectTabFunc(this.$.picklist_open.$.openqty_table);
		    break;
		case 'picked':
			this._multiSelectTabFunc(this.$$('#picklist_picked').$.pickedqty_table);
		    break;
		case 'issued':
			this._multiSelectTabFunc(this.$$('#picklist_issued').$.issuedqty_table);
		    break;
		case 'staged':
			this._multiSelectTabFunc(this.$$('#picklist_staged').$.stagedqty_table);
		    break;
		};
	},
	
	_multiSelectTabFunc: function(table){
		var filterRow = table.$$('#tableFilterRow');
		var tableHeader = $j(table).find('.tableheader');
		this.$.selected_record_count.value = table.selectedRecords.length;
		this._showMultiSelect = (table.selectedRecords.length > 0 );
		this._toggleFields(this._showMultiSelect, table, filterRow);
	},

	_toggleFields: function(isMultiSelect, table, filterRow){
		$j(this.$.picklist_content_selector.$.outerdiv).toggleClass('disableLink', isMultiSelect);
		$j(table).find('.tableactions').find('div').attr('hidden', isMultiSelect);
		$j(table).find('.filter-toggle').attr('disabled',isMultiSelect);
		$j(filterRow).find('maximo-text').attr('readonly', filterRow && isMultiSelect);
		$j(filterRow).find('maximo-button').attr('disabled', filterRow && isMultiSelect);
	},
	
	_exitMultiSelect: function(){
		switch(this.pickListTabName){
		case 'open':
			this.invuselineopencount = -1;
	        this._exitTabMultiSelect(this.$.picklist_open.$.openqty_table, this.$.picklist_open);
		    break;
		case 'picked':
			this.invuselinepickedcount = -1;
	        this._exitTabMultiSelect(this.$$('#picklist_picked').$.pickedqty_table, this.$$('#picklist_picked'));
		    break;
		case 'issued':
			this.invuselineissuedcount = -1;
	        this._exitTabMultiSelect(this.$$('#picklist_issued').$.issuedqty_table, this.$$('#picklist_issued'));
		    break;
		case 'staged':
			this.invuselinestagedcount = -1;
	        this._exitTabMultiSelect(this.$$('#picklist_staged').$.stagedqty_table, this.$$('#picklist_staged'));
		    break;
        default:
        	this.invuselineopencount = -1;
        	this._exitTabMultiSelect(this.$.picklist_open.$.openqty_table, this.$.picklist_open.$.invuselinecollection);
        	break;
		}
    },
    
    _exitTabMultiSelect: function(table, tabPage){
    	this._showMultiSelect = false;
    	table.selectAllRecords(false);
		this.batchAction = false;
		tabPage.completeOverlayFunc(tabPage);
    },
    
    _pickSelected: function(e){
    	var opentabpage = this.$.picklist_open;
    	var selectedInvuselines = opentabpage.$.openqty_table.selectedRecords;
    	this._tabSetRowRecord(opentabpage, selectedInvuselines, opentabpage.$.openqty_table);
    	if(selectedInvuselines.length){
    		opentabpage._pickQuantity(e);
    	}
    },

    _stageSelected: function(e){
    	switch(this.pickListTabName){
    	case 'open':
    		var opentabpage = this.$.picklist_open;
        	var selectedInvuselines = opentabpage.$.openqty_table.selectedRecords;
        	this._tabSetRowRecord(opentabpage, selectedInvuselines, opentabpage.$.openqty_table);
        	if(selectedInvuselines.length){
        		opentabpage._stageQuantity(e);
        	}
        	break;
    	case 'picked':
    		var pickedtabpage = this.$$('#picklist_picked');
        	var selectedInvuselines = pickedtabpage.$.pickedqty_table.selectedRecords;
        	this._tabSetRowRecord(pickedtabpage, selectedInvuselines, pickedtabpage.$.pickedqty_table);
        	if(selectedInvuselines.length){
        		pickedtabpage._stageQuantity(e);
        	}
        	break;
        default:
        	var opentabpage = this.$.picklist_open;
	    	var selectedInvuselines = opentabpage.$.openqty_table.selectedRecords;
	    	this._tabSetRowRecord(opentabpage, selectedInvuselines, opentabpage.$.openqty_table);
	    	if(selectedInvuselines.length){
	    		opentabpage._stageQuantity(e);
	    	}
	    	break;
    	}
    },
    
    _issueSelected: function(e){
    	switch(this.pickListTabName){
    	case 'open':
    		var opentabpage = this.$.picklist_open;
        	var selectedInvuselines = opentabpage.$.openqty_table.selectedRecords;
        	this._tabSetRowRecord(opentabpage, selectedInvuselines, opentabpage.$.openqty_table);
        	if(selectedInvuselines.length){
        		opentabpage._issueQuantity(e);
        	}
        	break;
    	case 'picked':
    		var pickedtabpage = this.$$('#picklist_picked');
        	var selectedInvuselines = pickedtabpage.$.pickedqty_table.selectedRecords;
        	this._tabSetRowRecord(pickedtabpage, selectedInvuselines, pickedtabpage.$.pickedqty_table);
        	if(selectedInvuselines.length){
        		pickedtabpage._issueQuantity(e);
        	}
        	break;
    	case 'staged':
    		var stagedtabpage = this.$$('#picklist_staged');
        	var selectedInvuselines = stagedtabpage.$.stagedqty_table.selectedRecords;
        	this._tabSetRowRecord(stagedtabpage, selectedInvuselines, stagedtabpage.$.stagedqty_table);
        	if(selectedInvuselines.length){
        		stagedtabpage._issueQuantity(e);
        	}
        	break;
    	default:
    		var opentabpage = this.$.picklist_open;
	    	var selectedInvuselines = opentabpage.$.openqty_table.selectedRecords;
	    	this._tabSetRowRecord(opentabpage, selectedInvuselines, opentabpage.$.openqty_table);
	    	if(selectedInvuselines.length){
	    		opentabpage._issueQuantity(e);
	    	}
	    	break;
    	}
    },

    _restockSelected: function(e){
    	switch(this.pickListTabName){
    	case 'picked':
	    	var pickedtabpage = this.$$('#picklist_picked');
	    	var selectedInvuselines = pickedtabpage.$.pickedqty_table.selectedRecords;
	    	this._tabSetRowRecord(pickedtabpage, selectedInvuselines, pickedtabpage.$.pickedqty_table);
	    	if(selectedInvuselines.length){
	    		pickedtabpage._restockQuantity(e);
	    	}
	    	break;
    	case 'staged':
    		var stagedtabpage = this.$$('#picklist_staged');
        	var selectedInvuselines = stagedtabpage.$.stagedqty_table.selectedRecords;
        	this._tabSetRowRecord(stagedtabpage, selectedInvuselines, stagedtabpage.$.stagedqty_table);
        	if(selectedInvuselines.length){
        		stagedtabpage._restockQuantity(e);
        	}
        	break;
    	default:
    		var pickedtabpage = this.$$('#picklist_picked');
	    	var selectedInvuselines = pickedtabpage.$.pickedqty_table.selectedRecords;
	    	this._tabSetRowRecord(pickedtabpage, selectedInvuselines, pickedtabpage.$.pickedqty_table);
	    	if(selectedInvuselines.length){
	    		pickedtabpage._restockQuantity(e);
	    	}
	    	break;
    	}
    },

    _returnSelected: function(e){
    	var issuedtabpage = this.$$('#picklist_issued');
    	var selectedInvuselines = issuedtabpage.$.issuedqty_table.selectedRecords;
    	this._tabSetRowRecord(issuedtabpage, selectedInvuselines, issuedtabpage.$.issuedqty_table);
    	if(selectedInvuselines.length){
    		issuedtabpage._issuedReturnFlyout(e);
    	}
    },

    _tabSetRowRecord: function(tabpage, selectedInvuselines, table){
    	tabpage.selectedRow = $j(table).find('.tablerow')[0];
    	if(selectedInvuselines.length){
    		tabpage.currInvuselineIndex = 0;
    		tabpage.rowRecord = selectedInvuselines[tabpage.currInvuselineIndex];
    		tabpage.batchAction = true;
    	}
    },
    
    _isPickAllowed: function(tabpagename){
    	return (tabpagename==='open');
    },

    _isReturnAllowed: function(tabpagename){
    	return (tabpagename==='issued');
    },

    _isRestockAllowed: function(tabpagename){
    	return (tabpagename==='open' || tabpagename==='issued')
    },

    _isStageAllowed: function(tabpagename){
    	return (tabpagename==='open' || tabpagename==='picked')
    },

    _removeSelected: function(e){
    	var opentabpage = this.$.picklist_open;
    	var selectedInvuselines = opentabpage.$.openqty_table.selectedRecords;
    	this._tabSetRowRecord(opentabpage, selectedInvuselines, opentabpage.$.openqty_table);
    	if(selectedInvuselines.length){
    		opentabpage._removeBatchInvuselines(e);
    	}
    }
});