/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18, 5737-M66
 * (C) Copyright IBM Corp. 2019,2021 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication, or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

Polymer({
	is: 'maximo-picklist-open',
	behaviors: [BaseComponent, ItemMaster],
	
	listeners : {
		'maximo-table-collection-sort' : 'collectionOrderByChanged',
		'item-detail': '_displayItemDetail',
		'table-action-clicked' : '_openStateActions',
		'iron-overlay-opened': '_positionOverlay',
		'display-next-invuseline':'_displayNextInvuseline'
	},
	
	properties : {
		attributeList: {
			type: String,
			value: 'fromstoreloc,itemnum,item.rotating,item.conditionenabled,item.lottype,quantity,frombin,refwo,returnedqty,openqty,issuedqty,stagedqty,pickedqty,description,rel.inventory.mxapiinventory{itemnum},invusenum,invuselinenum,invpicklist.siteid--invuselinesiteid,invuselineid,invpicklistnum,assetnum,invreserve.displaytaskid--displaytaskid,siteid,fromstoreloc,location,tositeid,tostoreloc,fromconditioncode,invreserve.wogroup--refwonum,invreserve.ponum--refponum,invreserve.mrnum--refmrnum,invreserve.requireddate--reqdate'
		},
	   	
    	openPLLinesOrderBy: {
			type: String,
			value: '%2Bitemnum'
		},
		
		overlayPosition: {
        	type: Object
        },
        
        msgIntervalSecs : {
    		type: Number,
    		value: 3,
    		readOnly: false
    	},
    	
    	currInvuselineIndex: {
    		type: Number,
    		value: 0
    	},
        
        batchAction : {
    		type: Boolean,
    		value: false
    	},
    	
    	invuselineAction : {
    		type: String,
    		value: null
    	},
		
		/**
		 * Display the actions button.
		 */
		actions:{
			type: Object,
			value: function(){
				return {
					header:{
						width: window.innerWidth/5+ 'px',
						flexGrow: '0',
						padding: '0px'
					},
					list:[
						{
							icon:'Maximo:Overflow',
							action: 'sractions',
							style: 'transform: rotate(90deg)',
							label: $M.localize('uitext', 'mxapiinvres', 'actions')
						}
					]
				};
			}
		}
	},
	
	_loadPickListOpenTable : function() {
		this._initOpenQtyTable();
		this._refreshCollection();
	},
	
	_refreshCollection: function() {
		var self = this;
		var invuselinecollection = this.$.invuselinecollection;
		invuselinecollection.additionalParams = ['oslc.where=openqty%3E0'];
		if(invuselinecollection){
			invuselinecollection.refreshRecords().then(function(){
				self.isCollectionRefreshed = true;
				self._setTableStyles();
	  		});
		}
	},
	
	_setTableStyles() {
		var table = this.$.openqty_table;
		if(table){
			var pos = table.$.tbody.getBoundingClientRect();
			table.maxHeight= ($j(window).innerHeight() - (pos.top + $j('#workscape_footer').height() + $j(window).scrollTop()))+'px';
		}		
	},
	/**
  	 * Initialization of table with columns
  	 */
  	_initOpenQtyTable: function(){
  		if(this.isTableInitialized) {
  			return;
  		}
  		var tableColumns = [{
				'attribute' : 'itemnum',
				'title' : $M.localize('uitext', 'mxapiitem', 'item'),
				'flexGrow': '1',
				'sortable' : true,
				'hyperlink' : true,
				'listener' : 'item-detail'
			},
			{
				'attribute' : 'description',
				'title' : $M.localize('uitext', 'mxapiitem', 'description'),
				'flexGrow': '2',
				'sortable' : true
			},
			{
				'attribute' : 'refwo',
				'title' : $M.localize('uitext', 'mxapiinvpicklist', 'wo'),
				'flexGrow': '1',
				'sortable' : true
			},
			{
				'attribute' : 'frombin',
				'title' : $M.localize('uitext', 'mxapistoreloc', 'bin'),
				'flexGrow': '1',
				'sortable' : true,
			},
			{
				'attribute' : 'quantity',
				'title' : $M.localize('uitext', 'mxapiinvpicklist', 'quantity'),
				'flexGrow': '1',
				'sortable' : true,
			},
			{
				'attribute' : 'openqty',
				'title' : $M.localize('uitext', 'mxapiinvpicklist', 'open'),
				'flexGrow': '1',
				'sortable' : true,
			},
			{
				'attribute' : 'pickedqty',
				'title' : $M.localize('uitext', 'mxapiinvpicklist', 'picked'),
				'flexGrow': '1',
				'sortable' : true,
			},
			{
				'attribute' : 'stagedqty',
				'title' : $M.localize('uitext', 'mxapiinvpicklist', 'staged'),
				'flexGrow': '1',
				'sortable' : true,
			},
			{
				'attribute' : 'issuedqty',
				'title' : $M.localize('uitext', 'mxapiinvpicklist', 'issued'),
				'flexGrow': '1',
				'sortable' : true,
			},
			{
				'attribute' : 'returnedqty',
				'title' : $M.localize('uitext', 'mxapiinvpicklist', 'returned'),
				'flexGrow': '1',
				'sortable' : true,
			}
		];
		this.set('openqtyColumn', tableColumns);
  		this.isTableInitialized = true;
  	},
  	
  	collectionOrderByChanged: function(e) {
		if(e.target && e.target.collection){
			e.target.collection.totalCount = -1;
			e.target.collection.refreshRecords();
		}
	},
	
	_displayItemDetail: function(e){
		$M.toggleWait(true);  	
    	var self=this;
    	$M.workScape.closest('#inventory').addContent('invItemList');
    	var itemcollection = $M.getGlobalResource('itemcollection');
    	var itemnum;
    	
    	if(e.detail.model.record){
    		itemnum=e.detail.model.record.itemnum;
    	}
    	
    	var maximoQbe = new MaximoQbe();
		maximoQbe.setEquals('itemnum',itemnum);
		var qbeObject = maximoQbe.toJSON();
		itemcollection.queryWhere = qbeObject;
		itemcollection.searchTermValue='';
		itemcollection.collectionUri='';
		itemcollection.refreshRecords().then(function(result){
			if(result.response){
				$M.setSubRoute('inventoryItems/itemDetail');
				self.fire('display-itempage',{'pagename':'itemDetail','record':result.response.member[0],'collection':itemcollection,'isRedirected': 'PickListDetail','redirectedRecord':self.plrecord,'pickListTabName':self.pickListTabName,'schema':itemcollection.schema});
				$M.toggleWait(false);  	
			}
		});		 
	},
	
	_collectionRefreshHandler: function(e) {
		var invUseLineOpenCount;
		if(e.detail.totalCount !== undefined){
			invUseLineOpenCount = e.detail.totalCount;
		}else {
			invUseLineOpenCount = e.target.totalCount;
		}
		this.openLineCount = invUseLineOpenCount;
		this.fire('collection-count-change', {'invuselineopencount':invUseLineOpenCount, 'action':this.openaction});
	},
	
	/**
	 * Hide the actions.
	 */
	_overFlowIcon : function(){
		this.$.openStateActionOverlay.close();
	},
	
	_openStateActions: function(e){
		this.batchAction = false;
		this.selectedRow = e.detail.source.closest('.tablerow');
		var pos = $j(e.detail.source).find('button').offset();
        var left =  pos.left - 180.5;
        var dockedPos = $j(this.$.openqty_table).find('#tableFilterRow').length? $j(this.$.openqty_table).find('#tableFilterRow')[0].getBoundingClientRect() : $j(this.$.openqty_table).find('.thead')[0].getBoundingClientRect();
        var rowPos = $j(e.detail.source).closest('.tablerow')[0].getBoundingClientRect();
        var top = pos.top - 19;
        if(rowPos.top < dockedPos.bottom){
        	top = pos.top;
        }
        this.overlayPosition = {'butonpos': pos,'left':left + 15,'top':top};
        this.setRowOffset = top;
        this.rowRecord = e.detail.record;
        this.$.openStateActionOverlay.open();
	},
	
	_positionOverlay: function(e){
		var windowHeight = $j(window).height() - 150;
		var pos = this.overlayPosition.butonpos;
        var topLastItems = pos.top + 50;
        if((windowHeight+$j(window).scrollTop()) < topLastItems){
            $j('.openStateActionsBottomDiv').addClass('add-pos-top');
            $j('.openStateActionTopDiv').addClass('add-remove-border');
        } else {
        	$j('.openStateActionsBottomDiv').removeClass('add-pos-top');
        	$j(this.$.openStateActionsBottomDiv).removeClass('adjust-top');
            $j('.openStateActionTopDiv').removeClass('add-remove-border');
        }
        $j(this.$.openStateActionOverlay).css({'left':this.overlayPosition.left,'top':this.overlayPosition.top});
	},

	_pickQuantity: function(e){
		e.preventDefault();
		e.stopPropagation();
		var self = this;
		this.openaction = 'Pick';
		this.$.openStateActionOverlay.close();
		var tableOffset;
		var openStateTable = $j(this.$.openqty_table).find('.table');
		var tableOffsetLeft = openStateTable.offset().left;
		var tableOffsetWidth = $j(openStateTable).innerWidth() - 8;
		if(window.navigator.userAgent.toLowerCase().indexOf('firefox') > -1){
			tableOffsetWidth = $j(openStateTable).innerWidth() - 16;
		}
		var pos = openStateTable.find('.thead')[0].getBoundingClientRect();
		tableOffset =  pos.bottom + $j(window).scrollTop();
		self.openFlyout = $M.showFlyout('pick_open', 'maximo-open-pick-flyout',  null, {'hideFooter':true ,'invuselineRecord': self.rowRecord, 'constrainWidth':true, 'top':tableOffset, 'flyoutOwner':self, 'batchPick':self.batchAction, 'invuselineIndex':self.currInvuselineIndex});
		self.async(function(){
			var flyoutDiv = $j(self.openFlyout.$.flyout);
			self._setFlyoutStyle(flyoutDiv, tableOffsetLeft, tableOffsetWidth, openStateTable, self.openFlyout);
		},300);
	},
	
	_issueQuantity: function(e){
		e.preventDefault();
		e.stopPropagation();
		var self = this;
		this.openaction = 'Issue';
		this.$.openStateActionOverlay.close();
		var tableOffset;
		var openStateTable = $j(this.$.openqty_table).find('.table');
		var tableOffsetLeft = openStateTable.offset().left;
		var tableOffsetWidth = $j(openStateTable).innerWidth() - 8;
		if(window.navigator.userAgent.toLowerCase().indexOf('firefox') > -1){
			tableOffsetWidth = $j(openStateTable).innerWidth() - 16;
		}
		var pos = openStateTable.find('.thead')[0].getBoundingClientRect();
		tableOffset =  pos.bottom + $j(window).scrollTop();
		self.openFlyout = $M.showFlyout('issue_open', 'maximo-open-issue-flyout',  null, {'hideFooter':true ,'invuselineRecord': self.rowRecord, 'constrainWidth':true, 'top':tableOffset, 'flyoutOwner':self, 'batchIssue':self.batchAction, 'invuselineIndex':self.currInvuselineIndex});
		self.async(function(){
			var flyoutDiv = $j(self.openFlyout.$.flyout);
			self._setFlyoutStyle(flyoutDiv, tableOffsetLeft, tableOffsetWidth, openStateTable, self.openFlyout);
		},300);
	},
	
	_stageQuantity: function(e){
		e.preventDefault();
		e.stopPropagation();
		var self = this;
		this.openaction = 'Stage';
		this.$.openStateActionOverlay.close();
		var tableOffset;
		var openStateTable = $j(this.$.openqty_table).find('.table');
		var tableOffsetLeft = openStateTable.offset().left;
		var tableOffsetWidth = $j(openStateTable).innerWidth() - 8;
		if(window.navigator.userAgent.toLowerCase().indexOf('firefox') > -1){
			tableOffsetWidth = $j(openStateTable).innerWidth() - 16;
		}
		var pos = openStateTable.find('.thead')[0].getBoundingClientRect();
		tableOffset =  pos.bottom + $j(window).scrollTop();
		self.openFlyout = $M.showFlyout('stage_open', 'maximo-open-stage-flyout',  null, {'hideFooter':true ,'invuselineRecord': self.rowRecord, 'constrainWidth':true, 'top':tableOffset, 'flyoutOwner':self, 'batchStage':self.batchAction, 'invuselineIndex':self.currInvuselineIndex});
		self.async(function(){
			var flyoutDiv = $j(self.openFlyout.$.flyout);
			self._setFlyoutStyle(flyoutDiv, tableOffsetLeft, tableOffsetWidth, openStateTable, self.openFlyout);
		},300);
	},
	
	_setFlyoutStyle: function(flyoutDiv, tableOffsetLeft, tableOffsetWidth, openStateTable, flyout) {
		if(flyout) {
			openStateTable.find('.tableheader:nth-child(9)').css('backgroundColor',  this.getComputedStyleValue('--Primary-green10'));
		}
		flyoutDiv.css({'left': tableOffsetLeft + 'px', 'width': tableOffsetWidth, 'box-sizing': 'border-box'});
		flyoutDiv.find('#internal').css({'padding': '1px 0 0','justify-content': 'left'});
		var navbar = $j('maximo-navbar')[0];
  		if(navbar){
  			var sideMenu = $j(navbar.$.sideMenu).find('.container').outerWidth() + 10 +'px';
  			flyoutDiv.find('#main_div').css('width', 'calc(100vw - '+sideMenu);
  		}
	},
	
	_displayInvUseLineDetail: function() {
		var	detailobj = [
			{
				'attribute' : 'assetnum',			
				'label' : [$M.localize('uitext','mxapiinvres', 'assetnum')],
				'optional': false
			},
			{
				'attribute' : 'displaytaskid',				
				'label' : [$M.localize('uitext','mxapiinvres', 'displaytaskid')],
				'optional': false
			},
			{
				'attribute' : 'invuselinesiteid',				
				'label' : [$M.localize('uitext','mxapiinvres', 'storelocsiteid')],
				'optional': false
			},
			{
				'attribute' : 'fromstoreloc',				
				'label' : [$M.localize('uitext','mxapiinvres', 'location')],
				'optional': false
			},
			{
				'attribute' : 'location',				
				'label' : [$M.localize('uitext','mxapiinvres', 'oplocation')],
				'optional': false
			},
			{
				'attribute' : 'fromconditioncode',				
				'label' : [$M.localize('uitext','mxapiinvres', 'conditioncode')],
				'optional': true
			},
			{
				'attribute' : 'tositeid',				
				'label' : [$M.localize('uitext','mxapiinvres', 'newsite')],
				'optional': false
			},
			{
				'attribute' : 'tostoreloc',				
				'label' : [$M.localize('uitext','mxapiinvres', 'tostoreloc')],
				'optional': false
			},
			{
				'attribute' : 'reqdate_localized',				
				'label' : [$M.localize('uitext','mxapiinvpicklist', 'requireddate')],
				'format':($M.maxauth && $M.maxauth.whoami ? $M.maxauth.whoami.dateformat.toUpperCase() : 'M/D/YY'),
				'optional': false
			},
			{
				'attribute' : 'refponum',				
				'label' : [$M.localize('uitext','mxapiinvres', 'ponum')],
				'optional': true
				
			},
			{
				'attribute' : 'refmrnum',				
				'label' : [$M.localize('uitext','mxapiinvres', 'mrnum')],
				'optional': true
				
			},
			{
				'attribute' : 'refwonum',
				'label' : [$M.localize('uitext', 'mxapiinvres', 'refworkorder')],
				'optional': true
			}
		]
		return detailobj;
	},
	
	_removeInvuseline: function(){
		var self = this;
		this.openaction = 'Remove';
		var removeAction = true;
		var invuselineData = {};
		this.$.openStateActionOverlay.close();
		var message = $M.localize('uitext','mxapiinvpicklist','invuselineremoved');
		var tableRow = $j('#pickListDetail_picklist_open_openqty_table_tablerow_' + this.rowRecord._id)[0];
		
		invuselineData.invusenum = this.rowRecord.invusenum;
		invuselineData.siteid = this.rowRecord.siteid;
		invuselineData.invuselinenum = this.rowRecord.invuselinenum;
		invuselineData.invpicklistnum = null;
		
		var invuselineopenresource = this.$.invuselineresource;
		invuselineopenresource.resourceUri = this.rowRecord.localref;
		if(tableRow){
			var removeoverLay =  $M.createOverlay(tableRow, message, null, function(){
				removeAction = false;
				if(this.isAuthorized('MXAPIINVPICKLIST:READ'))
					$j(removeoverLay).remove();
			},this.msgIntervalSecs, this.parentElement.parentElement, null, true);
		}
		
		this.async(function(){
			if(this.isAuthorized('MXAPIINVPICKLIST:READ'))
				$j(removeoverLay).remove();
	    	if(removeAction){
	    		this.onRemoveComplete(invuselineData, invuselineopenresource, this);
	    	}
		}, (this.msgIntervalSecs*1000));
	},
	
	onRemoveComplete: function(invuselineData, invuselineopenresource, self) {
		$M.toggleWait(true);
		invuselineopenresource.updateRecord(invuselineData, null, true, true);
	},

	_removeBatchInvuselines: function(e){
		var picklistArr = [], bulkInvuselines = [];
    	var picklistData = {}, invuselineData = {};
    	var requestHeaders = {};
    	var self = this;
		var removeAction = true;
    	var selectedInvuselines = this.$.openqty_table.selectedRecords;
    	var deletionMsg = $M.localize('uitext','mxapiinvpicklist','reservationsremoved');
    	var pldetailpage = $M.workScape.querySelector('#pickListDetail');
    	
    	this.openaction = 'Remove';
    	picklistData.href = this.plrecord.href;
    	picklistData.invuseline = [];
    	if(selectedInvuselines.length > 0){
    		selectedInvuselines.forEach(function(invuseline, index){
    			invuselineData = {};
    			invuselineData.href = invuseline.href;
    			invuselineData.invpicklistnum = null;
    			bulkInvuselines.push(invuselineData);
    		});
    	}
    	picklistData.invuseline = JSON.parse(JSON.stringify(bulkInvuselines));
    	picklistArr.push(picklistData);

		var invuselineopenresource = this.$.invuselineresource;
		invuselineopenresource.resourceUri = this.plrecord.href.slice(0, this.plrecord.href.indexOf('mxapiinvpicklist')+16);

		var undoCallback = null, statusnotification = null;
		$M.toggleWait(true);
		var undoCallback = function(){
			if(self.isAuthorized('MXAPIINVPICKLIST:READ') && statusnotification)
				$j(statusnotification).remove();
    		removeAction = false;
    		$M.toggleWait(false);
    	};	    	
    	statusnotification = $M.notify(deletionMsg, $M.alerts.info, null, undoCallback, false);

		this.async(function(){
			if(self.isAuthorized('MXAPIINVPICKLIST:READ') && statusnotification) {				
				$j(statusnotification).remove();
			}
	    	if(removeAction){
	    		self.removeBatch(picklistArr, invuselineopenresource, self);
	    	}
		}, (self.msgIntervalSecs*1000));
	},

	removeBatch: function(picklistArr, invuselineopenresource, self){
		var requestHeaders = {};
		requestHeaders.allornothing = true;
		requestHeaders.method = 'BULK';
		
		invuselineopenresource.updateRecord(picklistArr, null, true, true, requestHeaders);
	},

	completeOverlayFunc: function(self) {
		var collection = self.$.invuselinecollection;
		var pldetailpage = $M.workScape.querySelector('#pickListDetail');
		var pldetailresource = pldetailpage.$.picklistresource;
		if(pldetailpage.plrecord && pldetailpage.plrecord.invuseline_collectionref){
			collection.collectionUri = pldetailpage.plrecord.invuseline_collectionref;
            $M.toggleWait(true);
            collection.refreshRecords().then(function(){
				pldetailresource.resourceUri = pldetailpage.plrecord.href;
				pldetailresource.loadRecord().then(function(result){
					$M.toggleWait(false);
					pldetailpage.plrecord = result;
					pldetailpage.$.plRequiredDate.value = result ? moment(result.reqdatepicklist_localized, pldetailpage.dateformat).format(pldetailpage.dateformat): undefined;
				});
		    });
		}
	},
	
	/**
	 *Prevent scroll to top after collection refresh
	 */
	_preventScrollToTop: function(e){
        var ironList = $j('#pickListDetail_picklist_open_openqty_table iron-list')[0];
        if(ironList){
        	ironList.scrollOffset = ironList._scrollTop;
        }
    },
	
    _actionCallback: function(record){
    	var pldetailpage = $M.workScape.querySelector('#pickListDetail');
    	return (pldetailpage.plrecord.status_maxvalue !== "CLOSED");
   	 },
  	
  	_handleRecordUpdateError : function(e) {
 		$M.toggleWait(false);
 		var error = e.detail;
 		if(error && error.request && error.request.response && error.request.response.Error && error.request.response.Error.reasonCode!=='BMXAA7705E')
 			$M.notify(error.request.response.Error.message, $M.alerts.error);
 	},
 	
 	_handleRecordUpdateSuccess: function(e) {
 		this._preventScrollToTop();
 		if(!this.batchAction){
 			this.completeOverlayFunc(this);
 		}
 		else {
 			if(this.openaction==='Remove'){
 				var pldetailpage = $M.workScape.querySelector('#pickListDetail');
 				pldetailpage._exitMultiSelect();
 			} else {
 				this.openFlyout.querySelector('#internal').firstElementChild._closeFlyout();
 	 			this.openFlyout.querySelector('#internal').firstElementChild._handleBatchAction();
 			}
 		}
 	},
 	
 	_displayNextInvuseline: function(e){
    	var selectedInvuselines = this.$.openqty_table.selectedRecords;
    	var currentIndex = e.detail.invuselineIndex;
    	if(currentIndex <= selectedInvuselines.length){
    		this.currInvuselineIndex = currentIndex;
    		this.rowRecord = selectedInvuselines[this.currInvuselineIndex];
    		this.batchAction = true;
    		switch(e.detail.invuselineAction){
    		case 'Pick':
    			this._pickQuantity(e);
    		    break;
    		case 'Stage':
    			this._stageQuantity(e);
    			break;
    		case 'Issue':
    			this._issueQuantity(e);
    			break;
    		}
    	}
    },
    
    /**
     * Open lookup to list all reservation 
     */
    _addReservation : function(){
    	var location=this.plrecord ? this.plrecord.fromstoreloc: undefined;
		var lookUp='reservationLookup';
		var currentDate =  moment($M.getISODateString(new Date())).format($M.getMaxauth().whoami.dateformat.toUpperCase());
		var additionalParam = 'oslc.where=requireddate=%22'+currentDate+'%22 and location=%22'+location+'%22';
		$M.showLookup(lookUp, this, lookUp, '/oslc/os/mxapiinvres?savedQuery=RESERVATIONLIST', true, additionalParam, null, null,true);
    },
    
    /**
	 * Hide Add reservation Button when pick list status is closed
	 */
    _toggleButton : function(plrecord){
    	return plrecord.status_maxvalue==='CLOSED';
    }
    
});