/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18, 5737-M66
 * (C) Copyright IBM Corp. 2018,2021 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication, or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
Polymer( {

	is : 'maximo-identify',

	properties : {
		maximoMap : {
			type : Object,
			observer : '_mapChanged'
		},
		layers : {
			type : Array
		},
		urlLayersMap : {
			type : Object
		},
		maximoLayersMap : {
			type : Object,
		},
		mapname : {
			type : String,
			value : null
		},
		siteId : {
			type : String
		},
		baseUrl : {
			type : String,
			value : ''
		},
		postBody : {
			type : String
		},
		proxy : {
			type : String,
			value : ""
		},
		icon : {
			type : String
		},
		tool : {
			type : String,
			observer : '_toolSelected'
		},
		tolerance : {
			type : Number
		},
		pointMove : {},
		vectorLayer : {},
		featuresIdentified : {
			type : Array
		},
		currentFeature : {
			type : Object
		},
		totalFeatures : {
			type : Number
		},
		listFeatures: {
			type: Array
		},
		hideOpenIcon: {
			type : Boolean
		},
		drawerOpened: {
			type : Boolean
		},
		hasManyFeatures: {
			type : Boolean
		},
		pointerMoveHandler: {
			type: Object,
		},
		pointerMoveEvent: {
			type: Object,
		},
		pointerDownEvent: {
			type: Object,
		},
		delayedOnPointerMove: {
			type: Number,
		},
		pointerDownEventPoint: {
			type: Object,
		},
		pointerUpEvent: {
			type: Object,
		},
		pointerDragEvent: {
			type: Object,
		},
		pointerSlingleClickEvent: {
			type: Object,
		},
		mapTipsVectorLayers: {
			type: Object,
		},
		longPressStartTime: {
			type: Object
		}
	},
	ready : function () {
		this.toolId = "IdentifyTool"
		this.maximoLayersMap = {};
		this.hideOpenIcon = true;
		this.drawerOpened = null;
		this.hasManyFeatures = false;
		this.icon = $M.maxauth.baseUri
				+ '/webclient/javascript/pluss/com/esri/solutions/jsviewer/assets/pluss/iot18/toolbar/tb_identify.png';
		
		window.addEventListener('_identifyAddFeaturesToSidePanel', (evt)=> {
			//Add the features to the Side Panel
			this.featuresIdentified = evt.detail.features;
			this._loadSidePanelDialog();
		});
		this.explodedFeatures = [];
	},
	_mapChanged : function () {
		this.tolerance = this.maximoMap.properties.defaultTolerance;
		const container = Polymer.dom( this.root ).querySelector( '#identifyPopup' );
		const closer = Polymer.dom( this.root ).querySelector( '#identifyPopupCloser' );
		
		
		this.maximoMap.map.on( 'singleclick', ( evt )=> {
			if (this.maximoMap.identifyDisabled) {
				return;
			}
			this._handleMapClickForIdentify( evt.pixel, evt.coordinate );
		} );
		
		
		this._addHoverInteraction();
		this._enableMousePointTolerance();
	},
	
	_openIdentifyToleranceWindow: function() {
		this.$.identifyDialog.style[ "right" ] = "5px";
		this.$.identifyDialog.style[ "position" ] = "fixed";
		this.$.identifyDialog.style[ "top" ] = "55px";
		this.$.identifyDialog.open();
	},
	
	_closeMapTip: function() {
		console.log("Closing map tip");
	},
	
	_toolSelected : function ( e ) {
		if ( e != "" && e != null && e == this.toolId ) {
			this._openIdentifyToleranceWindow();
		}

	},
	closeDialog : function () {
		this.closeMapTipPanEvent();
		this.$.identifyDialog.close();
	},
	_enableMousePointTolerance : function () {

		//Create identify tolerance layer
		this.vectorLayer = new ol.layer.Vector( {
			source : new ol.source.Vector( {} )
		} );
		this.vectorLayer .set("layerName", "identifyLayer");
		
		this.vectorLayer.setZIndex(this.maximoMap.mapManager.maxZIndex + 1);
		
		this.maximoMap.map.addLayer( this.vectorLayer );
		
		//Create highlight layer
		this.highlightLayer = new ol.layer.Vector( {
			source : new ol.source.Vector( {} )
		} );
		this.highlightLayer.set("layerName", "highlightLayer");
		this.highlightLayer.setZIndex(this.maximoMap.mapManager.maxZIndex + 2);
		
		this.maximoMap.map.addLayer( this.highlightLayer );
		
		var self = this;
		this.pointMove = this.maximoMap.map.on( 'pointermove', function ( evt ) {
			
			var vectorSource = self.vectorLayer.getSource();
			vectorSource.clear();

			if (self.maximoMap.identifyDisabled) {
				return;
			}
			
			if (evt.dragging) {
				self.longPressStartTime = null;
			}
			
			var coordinate = evt.coordinate;
			
			var circlestyle = new ol.style.Style( {
				geometry : new ol.geom.Point( coordinate ),
				image : new ol.style.Circle( {
					radius : self.tolerance,
					fill : new ol.style.Fill( {
						color : 'rgba(0, 0, 0, 0.16)'
					} ),
					stroke : new ol.style.Stroke( {
						color : 'rgba(71, 71, 71, 0.54)',
						width : 1
					} )
				} ),
			} );

			var feature = new ol.Feature( new ol.geom.Circle( coordinate ) );
			feature.setStyle( circlestyle );
			vectorSource.addFeature( feature );
		} );
	},
	_disableMousePointTolerance : function () {
		ol.Observable.unByKey( this.pointMove );
		this.maximoMap.map.removeLayer( this.vectorLayer );
	},
	_loadSidePanelDialog : function ( pixel ) {
		var esriFieldTypeOIDs = [];
		var maximoKeys = [];

		this.totalFeatures = this.featuresIdentified.length;
		
		var listFeatures = [];
		
		this.featuresIdentified.forEach((feature)=> {
			let fieldName;
			let displayFieldNameValue;
			let value;
			
			/*
			 * If the displayFieldName attribute is not filled, use the esriFieldTypeOID
			 */
			if (feature.attributes) {
				
				fieldName = feature.esriFieldTypeOID;
				value = feature.attributes[fieldName].value;
				
				var featureAttribute = feature.attributes[feature.displayFieldName];
				var attributeValue = featureAttribute.value;
				if ( featureAttribute && attributeValue	&& attributeValue.trim() !== '' ) {
					let displayFieldName = feature.displayFieldName;
					displayFieldNameValue = attributeValue;
				} else {
					displayFieldNameValue = value;
				}				
				
				
			} else {
				fieldName = feature.maximoAttributes.maximoPrettyPrintKeyName;
				value = feature.maximoAttributes.maximoPrettyPrintKeyValue;
				displayFieldNameValue = value;
			}
			
			listFeatures.push(fieldName + '|' + value +':' + displayFieldNameValue);
		})
		
		this.set("listFeatures", listFeatures);
		
		if (this.totalFeatures > 0) {
			this._buildInfoWindow( 0 );
		} else {
			this.set('hideOpenIcon', true);
			this._hideIdentifyDialog();
		}
	},
	
	_onSelectedFeatureChange: function(e)
	{
		var value = e.target.value;
		
		console.log("Change ", value);
		var values = value.split('|');
		var attributeName = values[0];
		var attributeValue = values[1];
		
		var featureSelected = null;
		this.featuresIdentified.forEach((feature)=> {
			if ((feature.attributes && feature.attributes[attributeName] 
				&& feature.attributes[attributeName].value == attributeValue) 
					|| (feature.maximoAttributes && 
							feature.maximoAttributes.maximoPrettyPrintKeyValue == attributeValue)) {
				featureSelected = feature;
			}
		});
		this.currentFeature = featureSelected;
		if (featureSelected != null) {
			this._buildFeatureAttributesWindows(featureSelected);
		}
		
	
	},
	
	/*
	 * This method merge the features returned from Maximo with the features already stored in OpenLayers, if the
	 * feature is updated on Maximo side we need to make sure the local feature will be updated as well
	 */
	mergeFeatures: function(featuresFromMaximo, localFeatures) {
		var featuresResult = [];
		if (localFeatures.length == 0) {
			featuresResult = featuresFromMaximo;
		} else if (featuresFromMaximo.length == 0) {
			featuresResult = localFeatures;
		} else {
			// Do the merge
			localFeatures.forEach( (localFeature) => {
				let esriFieldTypeOID = localFeature.esriFieldTypeOID;
				let localFeatureOID = null;
				let localFeatureLayerId = localFeature.layerId;
				if (esriFieldTypeOID) {
					localFeatureOID = localFeature.attributes[esriFieldTypeOID];
				}
				let found = false;
				featuresFromMaximo.forEach( (featureFromMaximo) => {
					let featureFromMaximoOID = null;
					if (featureFromMaximo.attributes)
						featureFromMaximoOID = featureFromMaximo.attributes[esriFieldTypeOID];
					let featureFromMaximoLayerId = featureFromMaximo.layerId;
					if (featureFromMaximoOID == localFeatureOID && featureFromMaximoLayerId == localFeatureLayerId) {
						localFeature.maximoAttributes = featureFromMaximo.maximoAttributes; 
						localFeature.attributes = featureFromMaximo.attributes;
						found = true;
						featuresResult.push(localFeature);
					} else if ( featureFromMaximo.maximoAttributes && featureFromMaximo.maximoAttributes.isAutoLocated ) {
						localFeature.maximoAttributes = featureFromMaximo.maximoAttributes; 
						found = true;
					}
				})
				
				if (found == false) {
					featuresResult.push(localFeature);
				}
			})
		}
		return featuresResult;
	},
	
	_onToleranceChange: function() {
		if (this.mapTipsVectorLayers && this.pointerDownEventPoint) {
			var coordinate = this.maximoMap.map.getCoordinateFromPixel(this.pointerDownEventPoint.pixel);
			var vectorSource = this.mapTipsVectorLayers.getSource();
			vectorSource.clear();

			var feature = this._createHoverFeature(coordinate);
			vectorSource.addFeature( feature );
			
			this._searchFeatureForMapTips(this.pointerDownEventPoint.pixel, true);
		}
	},
	
	_buildFeatureAttributesWindows: function(feature) {
		const identifyContent = this.$.identifyContent;// Polymer.dom( this.root ).querySelector(
														// '#identifyPopupContent' );
		identifyContent.innerHTML = '';

		let gisAttributes = feature.attributes;
		if ( feature.attributes ) {
			identifyContent.innerHTML += "<span style='font-weight: bold'>" + feature.layerName + '</span><br/><hr/>';
			for ( let key in gisAttributes ) {
				if ( gisAttributes.hasOwnProperty( key ) ) {
					identifyContent.innerHTML += gisAttributes[ key ].alias + '= ' + gisAttributes[ key ].value + '<br/>';
				}
			}
			identifyContent.innerHTML += "<br/>";
		}
		let maximoAttributes = feature.maximoAttributes;
		if ( maximoAttributes != null ) {
			var mapTip = feature.maximoAttributes.mapTip;
			if ( feature.attributes )
				identifyContent.innerHTML += "<span style='font-weight: bold'>Maximo Attributes</span><br/><hr/><br/>";
			else
				identifyContent.innerHTML += "<span style='font-weight: bold'>"
						+ feature.maximoAttributes.mboNamePrettyPrint + "</span><br/><hr/><br/>";
			identifyContent.innerHTML += mapTip;
		}
		this._showIdentifyDialog();
		this._highlightFeature(feature);
	},
	
	_highlightFeature: function(feature) {
		
		if (feature == null) {
			return;
		}
		var esrijsonFormat = new ol.format.EsriJSON();
		
		var geometryType = feature.geometryType;
		var geometry = feature.geometry;
		
		var olFeature = esrijsonFormat.readFeature(feature);
		
		var style = null;
		
		// TODO Use this criteria to place the pinpoint
		// let featureCentroid = this.maximoMap.calculateFeatureCentroid( feature );
		
		switch(geometryType) {
			case 'esriGeometryPolyline':
			case 'esriGeometryLine':
				style = new ol.style.Style({
				       stroke : new ol.style.Stroke(
				    		   {
				    			   color : 'rgba(255,0,255,0.5)',
				    			   width: 5
				    		   })
				});
				
				break;
			case 'esriGeometryPoint':
				style = new ol.style.Style({
				      image: new ol.style.Circle({
				        radius: 7,
				        fill: new ol.style.Fill({color: 'rgba(255,0,255,0.5)'})
				      })
				    })
				break;
			case 'esriGeometryPolygon': 
				style = new ol.style.Style({
					   stroke : new ol.style.Stroke({
						   color : 'rgba(255,0,255,0.5)',
						   width : 2
						}),
					   fill : new ol.style.Fill({
						   color: 'rgba(255,0,255,0.5)'
					   })
					 });
				break;
		}
		
		var vectorSource = this.highlightLayer.getSource();
		vectorSource.clear();
		
		if (style != null) {
			olFeature.setStyle(style);
			vectorSource.addFeature(olFeature);
		}	
	},
	
	/**
	 * Handle for every Drawer action (open or close)
	 */
	_onDrawerChanged: function() {
		if (this.drawerOpened == null) {
			this.$.drawerComponent.openDrawer();
			this.$.drawerComponent.closeDrawer();
			this.drawerOpened = false;
		} else {
			if (!this.drawerOpened) { // Opening
				this.fire('_plussDrawerOpened', {opened: true, width: this.$.drawerComponent.drawerWidth});
				this._highlightFeature(this.currentFeature);
			} else { // closing
				this.fire('_plussDrawerOpened', {opened: false, width: this.$.drawerComponent.drawerWidth});
				if (this.highlightLayer) {
					var vectorSource = this.highlightLayer.getSource();
					vectorSource.clear();
				}
			}
			this.drawerOpened = !this.drawerOpened;
			if (this.featuresIdentified == null || this.featuresIdentified.length == 0) {
				this.set('hideOpenIcon', true);
			} else {
				this.set('hideOpenIcon', this.drawerOpened);
			}
		}
	},
	
	_showIdentifyDialog: function() {
		this.$.drawerComponent.openDrawer();
	},
	
	_hideIdentifyDialog: function() {
		this.$.drawerComponent.closeDrawer();
	},
	
	_buildInfoWindow : function ( featureNum ) {
		const feature = this.featuresIdentified[ featureNum ];
		this.currentFeature = feature;
		
		if (this.featuresIdentified.length > 1) {
			this.set('hasManyFeatures', true);
		} else {
			this.set('hasManyFeatures', false);
		}
		
		this._buildFeatureAttributesWindows(feature);
	},
	_handleMapClickForIdentify : function ( pixel, coordinate ) {

		this.featuresIdentified = [];
				
		let imageDisplay = this.maximoMap.map.getSize();
		let mapExtent = this.maximoMap.map.getView().calculateExtent( imageDisplay ).toString();
		const spatialReference = this.maximoMap.map.getView().getProjection().getCode().replace( /EPSG:/g, "" );

		imageDisplay = imageDisplay.toString();
		
		/*
		 * Creates an squared tolerance area to be used for the Auto located request and for some other linked features
		 * requests.
		 */
		let xmin = pixel[0] - this.tolerance;
		let xmax = pixel[0] + this.tolerance;
		let ymin = pixel[1] - this.tolerance;
		let ymax = pixel[1] + this.tolerance;
		let pixelMinCoordinate = this.maximoMap.map.getCoordinateFromPixel([xmin, ymin]);
		let pixelMaxCoordinate = this.maximoMap.map.getCoordinateFromPixel([xmax, ymax]);
		let toleranceExtentInXY = ol.extent.boundingExtent([pixelMinCoordinate, pixelMaxCoordinate]);
		
		/*
		 * Transforming the minimum point of the tolerance extent to lat/long projection.
		 */
		let toleranceExtentInLatLongMinimumPoint = ol.proj.transform( [
			toleranceExtentInXY[ 0 ], toleranceExtentInXY[ 1 ]
		], this.maximoMap.map.getView().getProjection(), 'EPSG:4326' );

		/*
		 * Transforming the maximum point of the tolerance extent to lat/long projection.
		 */
		let toleranceExtentInLatLongMaximumPoint = ol.proj.transform( [
			toleranceExtentInXY[ 2 ], toleranceExtentInXY[ 3 ]
		], this.maximoMap.map.getView().getProjection(), 'EPSG:4326' );

		/*
		 * We need to call the reverse() method because the Latitude corresponds to the y coordinate and the Longitude
		 * corresponds to the x coordinate.
		 */
		let toleranceExtentInLatLong = toleranceExtentInLatLongMinimumPoint.reverse().concat(
			toleranceExtentInLatLongMaximumPoint.reverse() );

		toleranceExtentInLatLong = toleranceExtentInLatLong.toString();
		toleranceExtentInXY = toleranceExtentInXY.toString();
		
		let basemapServiceName = this.maximoMap.getBasemapLayer().mapServiceName;
		let basemapURL = this.maximoMap.removeProxyURL(this.maximoMap.getLayerUrl(this.maximoMap.getBasemapLayer()));
		
		let self = this;

		const identifyPoint = coordinate[0] + "," + coordinate[1];
		const radius = Math.ceil(this.tolerance * this.maximoMap.map.getView().getResolution() * 
				this.maximoMap.map.getView().getProjection().getMetersPerUnit());
			
		/**
		 * Preparing a request to Identify service
		 */
		let params = {
				'extentInXY' : toleranceExtentInXY,
				'extentInLatLong' : toleranceExtentInLatLong,
				'mapName' : this.maximoMap.getMapManager().mapname,
				'spatialReference' : spatialReference,
				'imageDisplay' : imageDisplay,
				'mapExtent' : mapExtent,
				'identifyPoint' : identifyPoint,
				'tolerance' : this.tolerance,
				'radius' : radius
		};
		
		console.log( 'Requesting Identify service for extent ' + toleranceExtentInXY + ' using params: ', params );
		
		this.$.xhr.headers = {};
		this.$.xhr.contentType = 'application/json';
		this.$.xhr.url = this.$.myauthenticator.getBaseUri() + '/oslc/plussidentifyrecords';
		this.$.xhr.body = JSON.stringify( params );
				
		let identifiableLayers = this.maximoMap.getDynamicLayers();
		
		let identifiedFeaturesResponseCounter = 0;
		if (identifiableLayers.length > 0) {
			
			let request = this.$.xhr.generateRequest();
			
			request.completes.then( function ( response ) {
				let responseJson = response.response;
				let requestParamenters = responseJson.requestParameters;
				let identifyResponseResults = responseJson.identifyData.features;
				
				console.log( 'Identify for extent ' + requestParamenters.extentInXY + ' using params: ', 
						requestParamenters,' retrieved: ' + identifyResponseResults.length + ' features.' );
				
				if ( identifyResponseResults .length > 0 ) {
					Array.prototype.push.apply( self.featuresIdentified, identifyResponseResults );
				}
				
				self._loadSidePanelDialog( );
				
			}, function ( rejected ) {
				let request = rejected.request;
				let error = rejected.error;
				console.log( request + ' ' + error );
			} );
		} else {
			self._loadSidePanelDialog( );
		}
		
	},
	
	/**
	 * Add the hover interaction to the map
	 */
	_addHoverInteraction: function () {
		var map = this.maximoMap.map;
		
		let layerExists = this.maximoMap.map.getLayers().getArray().find(function(layer) {
			  return layer.get('layerName') == 'plussMapTipLayer';
			});
		
		if (!layerExists) {
			this.mapTipsVectorLayers = new ol.layer.Vector( {
				updateWhileInteracting: true,
				source : new ol.source.Vector( {} )
			} );
			this.mapTipsVectorLayers.set("layerName", "plussMapTipLayer");
			
			this.mapTipsVectorLayers.setZIndex(this.maximoMap.mapManager.maxZIndex + 1);
			
			this.maximoMap.map.addLayer( this.mapTipsVectorLayers );
			
		}
		
		var popup = document.getElementById('popup');
		if (popup) {
			var style = popup.style;
			style['display'] = 'table-cell';
		}
		
		//Used by non-mobile devices (mouse interaction)
		if (this.pointerMoveHandler == null) {
			this._addPointerMoveEvent();
		}
		
		// Used by mobile devices (touch interaction)
		if (this.pointerDownEvent == null) {
			this._addPointerDownEvent();
			this._addPointerUpEvent();
		}
		
	},
	
	/**
	 * Pointer move event - Find map tips when the mouse stops for a predetermined milliseconds on top of the features
	 */
	_addPointerMoveEvent: function() {
		var map = this.maximoMap.map;
		this.pointerMoveHandler = map.on('pointermove', (evt)=> {
			if (evt.dragging) {
		        return;
		    }
			this.pointerMoveEvent = evt;
			if (this.delayedOnPointerMove >= 0) {
		        clearTimeout(this.delayedOnPointerMove);
		    }
			this.delayedOnPointerMove = setTimeout((evt)=> {
				//Search for features
				this._searchFeatureForMapTips(this.pointerMoveEvent.pixel, false);
				
			}, this.maximoMap.mapManager.mapConf.mapTipMoveEndDelayProperty);
		 });
	},
	
	_searchFeatureForMapTips: function(pixel, isPointerDrag) {
		var map = this.maximoMap.map;
		let tolerance = this.tolerance/2;
		let featuresFound = [];
		let needToAddFeature = (feature) => {
			let needToAddFeature = true;
			if (feature.maximoAttributes) {
				
				let uid = feature.maximoAttributes.maximoObjectIdValue;
				featureExists = featuresFound.some(function(featureFound) {
					  return featureFound.maximoAttributes.maximoObjectIdValue == uid;
					});
				needToAddFeature = !featureExists;
			} else {
				needToAddFeature = false;
			}
			return needToAddFeature;
		};
		
		let newExplodedFeatures = [];
		
		map.forEachFeatureAtPixel(pixel, 
			(feature, layer)=> {	
				let singleFeature;
				if (feature.get("features")) {
					if (feature.get("features").length == 1) {
						singleFeature = feature.get("features")[0];
					} else if (isPointerDrag) {
						if (this.explodedFeatures.indexOf(feature) == -1) {
							this.explodeFeatures(feature, layer);
							feature.get("features").forEach( (feature) => {
								if (needToAddFeature(feature)) {
									featuresFound.push(feature);
								}
							}); 
						}
						newExplodedFeatures.push(feature);
					}
				} else {
					singleFeature = feature;
				}

				if (singleFeature && needToAddFeature(singleFeature)) {
					featuresFound.push(singleFeature);
				}	

			},

			{
				'layerFilter' : (layer)=> {
					return layer.isMaximoLayer;
			},
				'hitTolerance': tolerance
			}
		);
		
		if (isPointerDrag) {
			this.explodedFeatures.forEach( (feature) => {
				let isFeatureStillExploded = (newExplodedFeatures.indexOf( feature ) > -1);
				if (!isFeatureStillExploded) {
					this.resetExplodedFeature(feature);
				}
			});
			this.explodedFeatures = newExplodedFeatures;
			
		}		

		let layer;
		
		if (isPointerDrag) {
			layer = this.maximoMap.map.getLayers().getArray().find(function(layer) {
			  return layer.get('layerName') == 'plussMapTipLayer';
			});
		} else {
			layer = this.maximoMap.map.getLayers().getArray().find(function(layer) {
				  return layer.get('layerName') == 'identifyLayer';
				});
		}
		
		let vectorSource = layer.getSource();
		let features = vectorSource.getFeatures();
		let feature = features[0];
		if (feature) {
			let extent = feature.getGeometry().getExtent();
	    	centerPoint = ol.extent.getCenter(extent);
	    	
			this._showFeatureMapTip(featuresFound, centerPoint);
		}
		
	},
	explodeFeatures: function(feature,layer) {
		const clusterDistance = layer.getSource().getDistance();
		const temporaryLayer = MxLayer.retrieveTemporaryMaximoLayer(layer.get("layerName"));
			
		const explodeStyleFunction = MxLayer.generateExplodeClusterFunction(clusterDistance);
		
		feature.setStyle(explodeStyleFunction(feature)[0]);
		
		this.explodedFeatures.push(feature);
		
		MxLayer.handleExplodedClusterSelectedFeature(feature, temporaryLayer);
	},
	resetExplodedFeature: function (feature) {
		const temporaryLayer = MxLayer.retrieveTemporaryMaximoLayerFromFeature(this.maximoMap.map, feature);
		feature.setStyle(null);
		const internalFeatures = feature.get("features") || [];
		internalFeatures.forEach( (feature) => {
			const featureId = feature.getId();
			const clonedFeature = temporaryLayer.getSource().getFeatureById("temporary-"+featureId);
			if (clonedFeature) 
				temporaryLayer.getSource().removeFeature(clonedFeature);
		});
		const extentFeature =  temporaryLayer.getSource().getFeatureById("extent-" + feature.ol_uid);
		if (extentFeature) 
			temporaryLayer.getSource().removeFeature(extentFeature);
		
	},
	/**
	 * Pointer drag event - Used to keep the hover feature fixed when the Long press mechanism is used
	 */
	_addPointerDragEvent: function() {
		var map = this.maximoMap.map;
		this.pointerDragEvent = map.on( 'pointerdrag', ( evt )=> {						
			var coordinate = this.maximoMap.map.getCoordinateFromPixel(this.pointerDownEventPoint.pixel);
			var vectorSource = this.mapTipsVectorLayers.getSource();
			vectorSource.clear();
			var feature = this._createHoverFeature(coordinate);
			vectorSource.addFeature( feature );
			this._searchFeatureForMapTips(this.pointerDownEventPoint.pixel, true);
		} );
	},
	
	
	/**
	 * Pointer up after a long press
	 */
	_addPointerUpEvent: function() {
		var map = this.maximoMap.map;
		this.pointerUpEvent = map.on('pointerup', (evt)=> {
			let endTime = new Date().getTime();
			
			if (this.longPressStartTime != null) {
				//Long press executed
				if (endTime - this.longPressStartTime > this.maximoMap.mapManager.mapConf.mapTipLongPressDelayProperty) {
					this.maximoMap.longPress = true;
					this.maximoMap.identifyDisabled = true;
					ol.Observable.unByKey( this.pointerMoveHandler );
					this.pointerMoveHandler = null;
					var coordinate = this.pointerDownEventPoint.coordinate;
					var vectorSource = this.mapTipsVectorLayers.getSource();
					vectorSource.clear();

					var feature = this._createHoverFeature(coordinate);
					vectorSource.addFeature( feature );
					
					
					var identifyLayer = map.getLayers().getArray().find(function(layer) {
					  return layer.get('layerName') == 'identifyLayer';
					});
					if (identifyLayer) {
						identifyLayer.getSource().clear();
					}
					
					ol.Observable.unByKey( this.pointerDownEvent );
					ol.Observable.unByKey( this.pointerUpEvent );
					this.pointerDownEvent = null;
					this.pointerUpEvent = null;
					
					this._addPointerDragEvent();
					this._openIdentifyToleranceWindow();
				} 
			} 
			
			
		});	
	},
	/**
	 * Pointer down - used to determine whether is a long press or not
	 */
	_addPointerDownEvent : function() {
		var map = this.maximoMap.map;
		this.pointerDownEvent = map.on('pointerdown', (evt)=> {
			this.maximoMap.longPress = false;
		    this.pointerDownEventPoint = evt;
		    this.longPressStartTime = new Date().getTime();
		});
	},
	
	/**
	 * Method to create the Hover feature to use as the reference to find Maximo features
	 */
	_createHoverFeature: function(coordinate, ) {
		var circlestyle = new ol.style.Style( {
			geometry : new ol.geom.Point( coordinate ),
			image : new ol.style.Circle( {
				radius : this.tolerance,
				fill : new ol.style.Fill( {
					color : 'rgba(0, 0, 0, 0.16)'
				} ),
				stroke : new ol.style.Stroke( {
					color : 'rgba(71, 71, 71, 0.54)',
					width : 1
				} )
			} )
		} );

		var feature = new ol.Feature( new ol.geom.Circle( coordinate ) );
		feature.setStyle( circlestyle );	
		return feature;
	},
	
	/**
	 * Close the map tip event when using Long Press and reset the events
	 */
	closeMapTipPanEvent: function() {
		this.maximoMap.identifyDisabled = false;
		ol.Observable.unByKey( this.pointerDragEvent );
		ol.Observable.unByKey( this.pointerSlingleClickEvent );
		this.pointerDragEvent = null;
		this.pointerSlingleClickEvent  = null;
		this.pointerDownEventPoint = null;
		var vectorSource = this.mapTipsVectorLayers.getSource();
		vectorSource.clear();
		let overlay = this.maximoMap.map.getOverlayById('mouseHoverOverlay');
		overlay.features = null;
    	overlay.setPosition(null);
		this._addHoverInteraction();
	},

	/**
	 * Show the Map Tip for a set of features
	 */
	_showFeatureMapTip: function(features, centerPoint) {
		if (features == null || features.length == 0) {
			return;
		}
		
		let mapTipValue, feature;
		// If there is just one feature, use the Map Tip Summary
		if (features.length == 1) {
			feature = features[0];
			let extent = feature.getGeometry().getExtent();
	    	centerPoint = ol.extent.getCenter(extent);
	    	mapTipValue = feature.maximoAttributes.mapTipSummary;
			
		} else {
			// If there are more than one feature, use the Mouse Over Map Tip
			mapTipValue = '';
			features.forEach((feature, i) => {
				mapTipValue += feature.maximoAttributes.mouseOver;
				if (i < features.length-1) {
					mapTipValue += "<hr/>";
				}
			});
		}
		
		let container = document.getElementById('popup-content');
    	container.innerHTML = mapTipValue;
    	let overlay = this.maximoMap.map.getOverlayById('mouseHoverOverlay');
    	
    	if (centerPoint) {
			let overlay = this.maximoMap.map.getOverlayById('mouseHoverOverlay');
			overlay.features = features;
	    	overlay.setPosition(centerPoint);
	    }
	},

} );
