/* IBM Confidential
 *
 * OCO Source Materials
 *
 * 5724-U18, 5737-M66
 *
 * (C) COPYRIGHT IBM CORP. 2011,2023
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
define(["dojo/main", "dijit/main", "dojox/main",  "dojo/io/script"], function(dojo, dijit, dojox, script) {
	
	/*
	* Set the mxmap global object
	*/
	if (!ibm.tivoli.fwm) {
		ibm.tivoli.fwm = {};
	}
	if (!ibm.tivoli.fwm.mxmap) {
		ibm.tivoli.fwm.mxmap = {};
	}
	if (!ibm.tivoli.fwm.mxmap.stack) {	
		ibm.tivoli.fwm.mxmap.stack = {};
	}
	
	/*
	 * factory will be available globally
	 */
	ibm.tivoli.fwm.mxmap.factory =  {
		events: {
			mapImplLoaded: "mxmap.mapImplLoaded"
		},
		dojoToDomId: null,
		esriVersion: "3.42",
		options: null,
		initialized: false,
		readyToCreateMap: false,
		tries: 10,
		/**
		 * In IE we need to first load MXN standard files and only after we can load
		 * the other ones. Otherwise IE will break complaining MXN object is not
		 * defined.
		 */

		/**
		 * Loads the core Mapstraction javascripts
		 */
		init: function()
		{
			// must be loaded first otherwise it breaks the other

			console.log("[Factory] is Ie?", (dojo.isIE != null));

			this.initialized = true;
			this.registry = {};
			console.log("[Factory] Initiated factory");
		},

		/**
		 * This method creates the provider implementation and triggers its
		 * createMap method
		 */
		createMap: function(options)
		{
			console.log("[Factory] Component ID: " + options.compId);
			console.log("[Factory] Is error in Map Configuration? " + options.mapConf.error);
			if (options.mapConf.error && options.mapConf.error == true)
			{
				if (options.mapConf.showErrorObject && options.mapConf.showErrorObject == true) {
					var params = {
							msgKey : options.mapConf.key,
							msgGroup : options.mapConf.group,
							params: options.mapConf.params
					};
					console.log("[Factory] Errors param: ", params);

					sendEvent('showErrorsParamObject', options.compId, JSON.stringify(params));	
				} else {
					console.log("[Factory] Options.mapConf.key: ", options.mapConf.key);
					addCommInput('msgKey', options.mapConf.key);
					addCommInput('msgGroup', options.mapConf.group);
					sendEvent('showErrors', options.compId, options.mapConf.key);
				}

				return;
			}
			if (this.initialized != true)
			{
				this.init();
			}

			if (this.mapExists(options.compId))
			{
				console.log("[Factory]  Map exists, destroying", options.compId);
				this.destroyCurrentMap(options.compId);
			}

			var providerName = options.mapConf.provider;
			var providerImplFactory = "ibm/tivoli/fwm/mxmap/factories/" + providerName;

			this.options = options;

			var me = this;

			if (providerName == "spatial") {
				var esriDojoConfig = this.configureSpatialProvider(options);

				var maxDojo = dojo;
				var maxDijit = dijit;
				var maxDojox = dojox;
				var maxDojoConfig = dojo.config;

				//when Maximo dialogs are opened, we will use Maximo Dojo
				//not necessary for Map Manager and Open Map
				if (!options.mapConf.isInMapManager && options.compId.indexOf("open_map") == -1) {
					this.openMaximoDialogHandler = dojo.subscribe("showdialog", function(event) {
						document.__dojoToDomId = ibm.tivoli.fwm.mxmap.factory._dojoToDomId;
						require({map: {"*": {dojo: "dojo", dijit: "dijit", dojox: "dojox"}}, async: false}, ["require"]);
						   console.log("Using Maximo Dojo");
					});
				}
				//use setTimeout with at least 200ms delay, as there are some events that will be fired by core dojo 200ms after the page is loaded and have to use core dojo
				setTimeout( function() {
					//Need to require this module in non AMD way, so already requiring it before using esri Dojo
					dojo.require("dojox.grid.enhanced._Plugin");
					require(esriDojoConfig,["dojo/has"], function(dojoHas) {

						if (dojoHas("touch")) {
							dojoHas.add("touch", function(global, document, element) {
								return false;
							}, true, true);
						}
						require(esriDojoConfig,[providerImplFactory, "ibm/tivoli/fwm/mxmap/impl/MaximoSpatial"], function(factory, MaximoSpatial) {
						
							dojo.config.fwm = maxDojoConfig.fwm;
							//Make global dojo point to the Maximo dojo again
							window.dojo = maxDojo;
							window.dijit = maxDijit;
							window.dojox = maxDojox;
							
							/*
							 * __dojoToDomId is set when creating dojo widgets
							 * as a new dojo version is loaded in Spatial, we need to clean this document property
							 * in case any widget was created using maximo dojo
							*/
							
							if (document.__dojoToDomId) {
								me._dojoToDomId = document.__dojoToDomId;
								document.__dojoToDomId = null;
							}

							options.maxDojoFwmConfig = maxDojoConfig.fwm;
							factory.init(options);
							console.log("[Factory]  Provider initiated.");
							var maximoSpatialMapImpl = new MaximoSpatial();
							var mapDojo = maximoSpatialMapImpl.selectDojoVersion();
							maximoSpatialMapImpl.createMap(options);
							//Add instance to the factory registry
							me.registry[options.compId] = {
									currentMap: maximoSpatialMapImpl
							};
							// Redirect core dojo events to spatial dojo
							var mapId = maximoSpatialMapImpl.getId();
							var mapFullScreenModeChangedEvent = "mapFullScreenModeChanged_" + mapId;
							me.openMaximoDialogHandler = maxDojo.subscribe(mapFullScreenModeChangedEvent, function(event) {
								mapDojo.publish(mapFullScreenModeChangedEvent, [event]);
							});
							window.dojo.publish(me.events.mapImplLoaded, [ factory, providerName ]);
						});
					});
				}, 201);
				
			} else {
				if (!this.loaded[providerName]) {
					ibm.tivoli.fwm.mxmap.stack[providerName] = [];
					ibm.tivoli.fwm.mxmap.stack[providerName].push(options);
					this.loaded[providerName] = -1;
					require([providerImplFactory], function(factory) {
						factory.init(options);
						console.log("[Factory]  Provider initiated.");
					});
				} else {
					if (this.loaded[providerName] == -1)
					{
						console.log("[Factory] Api is NOT loaded, queueing options");
						ibm.tivoli.fwm.mxmap.stack[providerName].push(options);
					}
					else
					{
						console.log("[Factory] Api is loaded");
						dojo.publish(this.events.mapImplLoaded, [ this.loaded[providerName], providerName ]);
						this._createProviderMap(options, this.loaded[providerName]);
					}
				}
			}
		},
		/**
		 * 
		 * Set the property map readyToCreateMap
		 * Used when the map creation has to wait for something else to load
		 */
		setReadyToCreateMap: function(isReadyToCreate) {
			this.readyToCreateMap = isReadyToCreate;
		},
		mapExists: function(compId)
		{

			return this.registry[compId] && this.registry[compId].currentMap;
		},
		destroyCurrentMap: function(compId)
		{
			console.log("[Factory] Destroying map", compId, this.registry[compId]);
			try
			{
				if (this.readyToCreateMapHandle) {
					dojo.unsubscribe(this.readyToCreateMapHandle);
				}
				if (this.openMaximoDialogHandler) {
					this.openMaximoDialogHandler.remove();
					this.openMaximoDialogHandler = null;
				}
				if (this.registry[compId] && this.registry[compId].currentMap)
				{
					this.registry[compId].currentMap.destroyRecursive();
				}
			}
			catch (e)
			{
				console.log("[Factory]  Could not destroy", e);
			}
			this.registry[compId] = null;
		},
		loaded: {},
		/**
		 * When the provider api is initializes this method is called.
		 */
		apiInitialized: function(mapImpl, provider)
		{
			console.log("[Factory] Loaded: ", mapImpl, provider);
			this.loaded[provider] = mapImpl;
			dojo.publish(this.events.mapImplLoaded, [ mapImpl, provider ]);
			this.executeStack(mapImpl, provider);
		},
		/**
		 * publish an event from Maximo 
		 * If the provider is Spatial, we need to publish the event from Esri Dojo
		 */
		publishMapEvent: function(compId, eventType, eventParams) {
			if (this.registry && this.registry[compId]) {
				var mapImpl = this.registry[compId].currentMap;
				if (mapImpl.providerName === 'maximospatial') {
					mapImpl.publishMapEvent(eventType, eventParams);
				} else {
					dojo.publish(eventType, eventParams);
				}
			}
		},
		executeStack: function(mapImpl, provider)
		{
			if (ibm.tivoli.fwm.mxmap.stack[provider])
			{
				var queuedRequest = ibm.tivoli.fwm.mxmap.stack[provider];

				for ( var id = 0; id < queuedRequest.length; id++)
				{
					var options = queuedRequest[id];
					this._createProviderMap(options, mapImpl);
				}
			}
		},
		_createProviderMap: function(options, mapImpl)
		{
			console.log("[Factory] Create: ", mapImpl, options);
			var instance = null;
			eval("this._instance = new " + mapImpl + "();");
			instance = this._instance;
			console.log("[Factory] Instance: ", instance);
			instance.createMap(options);
			this.registry[options.compId] = {
					currentMap: instance
			};
		},
		/**
		 * Configures the dojo paths and return an object with the esri dojo configuration
		 * @param option the map options
		 * @return an object with the esri dojo configuration 
		 */
		configureSpatialProvider: function(options) {

			var mobileString = '';
			var esriDojoConfig = {};

			options.esriVersion = this.esriVersion;

			var javascriptRelativePath = "../../";
			var javascriptFullPath = dojo.config.fwm.ctxRoot + '/webclient/javascript/'
			var esriApiDirectoriesPath = 'pluss/arcgis_js_api/library/' + this.esriVersion + '/' + this.esriVersion +  mobileString;
			var esriRelativePath = javascriptRelativePath + esriApiDirectoriesPath;
			var esriFullPath = javascriptFullPath + esriApiDirectoriesPath;

			window.URL_ESRI = esriFullPath + '/esri';
			window.URL_ESRI_DOJO = esriFullPath + "/dojo";
			window.URL_IBM = javascriptFullPath + 'ibm';

			var isInSchedulerApp = options.mapConf.currentMbo && options.mapConf.currentMbo.mxdata && options.mapConf.currentMbo.mxdata.mboName === "SKDPROJECT";
			var isInOpenMap = options.compId.indexOf("open_map") > -1;
			var isInMapManager = options.mapConf.isInMapManager;
			if (!(isInSchedulerApp || isInOpenMap || isInMapManager) ) {
				esriDojoConfig.async = true;
			}
			
			var esriPackages = [{
				name: "esri",
				location: esriRelativePath + "/esri"
			},
			{
				name: "esriDojo",
				location: esriRelativePath + "/dojo"
			},
			{
				name: "esriDijit",
				location: esriRelativePath + "/dijit"
			},
			{
				name: "esriDojox",
				location: esriRelativePath + "/dojox"
			},
			{
				name: "moment",
				location: esriRelativePath + "/moment"
			}
			];
			
			esriDojoConfig.packages = esriPackages.concat(require.rawConfig.packages);
				
			var esriDojoMap = {
				"dojo": "esriDojo",
				"dijit": "esriDijit",
				"dojox": "esriDojox"
			};
			
			esriDojoConfig.map = {
					"*": esriDojoMap
			};

			return esriDojoConfig;
		}
	}
	
	return ibm.tivoli.fwm.mxmap.factory;
});
